/*
 * Efficiently reads the duration of the high voltage in a pulse train.
 * Designed to simplify the use of RC controllers with the Arduino.
 *
 * Depends on the EnableInterrupt library.
 *
 * Author: David Wang, NuVu Studio
 */
 #include <Arduino.h>
 
/*
 * Behaves similarly to the built-in Arduino map function, but maps a "deadband" section of the input range to middle value of the output range.
 * 
 * The return value is constrainted to lie within the range "out_min" to "out_max".
 * The range of x from "in_dead_min" to "in_dead_max" is mapped to the output "out_mid".
 * The sequence of arguments "in_min", "in_dead_min", "in_dead_max", and "in_max" must monotonically increase or decrease.
 * The sequence of arguments "out_min", "out_mid", and "out_max" must monitonically increase or decrease.
 */
long deadbandMap(long x, long in_min, long in_dead_min, long in_dead_max, long in_max, long out_min, long out_mid, long out_max)
{
  if(in_min>in_max){
    long temp = out_max;
    out_max = out_min;
    out_min = temp;
    temp = in_max;
    in_max = in_min;
    in_min = temp;
    temp = in_dead_max;
    in_dead_max = in_dead_min;
    in_dead_min = temp;
  }
  if(x<in_min){
    return out_min;
  }else if(x<in_dead_min){
    return map(x,in_min,in_dead_min,out_min,out_mid);
  }else if(x>in_max) {
    return out_max;
  }else if(x>in_dead_max){
    return map (x,in_dead_max,in_max,out_mid,out_max);
  }else{
    return out_mid;
  }
}

/*
 * Behaves similarly to the built-in Arduino map function, but maps a "deadband" section of the input range to middle value of the output range.
 * 
 * The return value is constrainted to lie within the range "out_min" to "out_max".
 * The range of "deadband" values around the cente rof "in_min" and "in_max" are mapped to the output "out_mid".
 */
long deadbandMap(long x, long in_min, long in_deadband, long in_max, long out_min, long out_mid, long out_max)
{
  long in_double_mid = in_min+in_max;
  long in_dead_min;
  long in_dead_max;
  if((in_min*2)<in_double_mid){
    in_dead_min = (in_double_mid-in_deadband)/2;
    in_dead_max = (in_double_mid+in_deadband)/2;
  }else{
    in_dead_min = (in_double_mid+in_deadband)/2;
    in_dead_max = (in_double_mid-in_deadband)/2;
  }
  return deadbandMap(x,in_min,in_dead_min,in_dead_max,in_max,out_min,out_mid,out_max);
}